/// <reference types="@sveltejs/kit" />
// REMOVED: /// <reference types="@types/workbox-sw" />
/// <reference lib="webworker" />

// Import SvelteKit-provided variables ONLY
import { build, files, version } from '$service-worker';

declare let self: ServiceWorkerGlobalScope;
// Declare 'workbox' as any for now IF TypeScript still complains after removing @types/workbox-sw.
// Often, SvelteKit's types are enough, but this is a fallback.
declare const workbox: any; // Uncomment this line ONLY if 'Cannot find name workbox' persists

console.log(`[Service Worker] Version: ${version}`);

// --- Precaching ---
// Use the global workbox object (assuming SvelteKit injects it)
workbox.precaching.precacheAndRoute(build);
workbox.precaching.precacheAndRoute(files.map(f => ({ url: f, revision: null })));

// --- Runtime Caching ---
// Google Fonts
workbox.routing.registerRoute(
    ({ url }) => url.origin === 'https://fonts.googleapis.com' || url.origin === 'https://fonts.gstatic.com',
    new workbox.strategies.StaleWhileRevalidate({
        cacheName: 'google-fonts',
        plugins: [
            new workbox.cacheableResponse.CacheableResponse({ statuses: [0, 200] }),
            new workbox.expiration.ExpirationPlugin({ maxEntries: 20, maxAgeSeconds: 30 * 24 * 60 * 60 }),
        ],
    }),
);

// Images from origin
workbox.routing.registerRoute(
    ({ request, url }) => !!request && request.destination === 'image' && url.origin === self.location.origin,
    new workbox.strategies.CacheFirst({
        cacheName: 'images',
        plugins: [
            new workbox.cacheableResponse.CacheableResponse({ statuses: [0, 200] }),
            new workbox.expiration.ExpirationPlugin({
                maxEntries: 50,
                maxAgeSeconds: 30 * 24 * 60 * 60,
                purgeOnQuotaError: true,
            }),
        ],
    }),
);

// --- Lifecycle ---
self.addEventListener('install', (event) => {
    console.log('[Service Worker] Install event');
    // event.waitUntil(self.skipWaiting());
});

self.addEventListener('activate', (event) => {
    const extendableEvent = event as ExtendableEvent;
    console.log('[Service Worker] Activate event');
    extendableEvent.waitUntil(workbox.precaching.cleanupOutdatedCaches());
    // event.waitUntil(self.clients.claim());
});

self.addEventListener('fetch', (event) => {
    // console.log(`[Service Worker] Fetching: ${event.request.url}`);
});